#include "status.h"
#include <qthing.h>

#include "io.h"

#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "driver/gpio.h"
#include "esp_log.h"

bool status_led_enabled = false;
bool status_led_negated = false;
gpio_num_t status_led_gpio;

volatile connection_status_t connection_status;

TaskHandle_t status_led_task_handle = NULL;

void set_status_led(uint32_t level) {
    if (status_led_enabled) {
        ESP_ERROR_CHECK(gpio_set_level(status_led_gpio, level ^ status_led_negated));
    }
}

void show_activity() {
    if (status_led_task_handle != NULL) {
        xTaskNotify(status_led_task_handle, 0, eNoAction);
    }
}

void status_led_task(void* arg) {
    bool blink_state = false;
    while (true) {
        if (connection_status == connecting) {
            set_status_led(1);
            xTaskNotifyWait(0, 0, NULL, portMAX_DELAY);
        }
        else if (connection_status == disconnected) {
            set_status_led(blink_state);
            blink_state = !blink_state;
            vTaskDelay(1000 / portTICK_PERIOD_MS);
        }
        else {
            set_status_led(1);
            vTaskDelay(50 / portTICK_PERIOD_MS);
            set_status_led(0);
            xTaskNotifyWait(0, 0, NULL, portMAX_DELAY);
        }
    }
}

void enable_status_led(gpio_num_t gpio, bool negated) {
    if (status_led_enabled) {
        ESP_LOGE("STATUS_LED", "Status led is already enabled");
        return;
    }
    status_led_enabled = true;
    status_led_negated = negated;
    status_led_gpio = gpio;
    gpio_init_output(gpio, true);

    xTaskCreate(status_led_task, "status_led_task", 1024, NULL, 6, &status_led_task_handle);
    add_combined_mqtt_connection_status_callback([](connection_status_t status){
        connection_status = status;
        xTaskNotify(status_led_task_handle, 0, eNoAction);
    });
}
